/* @import url('https://fonts.googleapis.com/css2?family=IBM+Plex+Sans:ital,wght@0,400;0,500;0,600;0,700;1,400;1,500;1,600;1,700&display=swap'); */
@import url('https://fonts.googleapis.com/css2?family=IBM+Plex+Serif:ital,wght@0,400;0,500;0,600;0,700;1,400;1,500;1,600;1,700&display=swap');
@import url('https://fonts.googleapis.com/css2?family=IBM+Plex+Mono:ital,wght@0,400;0,500;0,600;0,700;1,400;1,500;1,600;1,700&display=swap');

@media all {
    body, input, div.sphinxsidebar input {
        /* font-family: "IBM Plex Sans", -apple-system, BlinkMacSystemFont, "Segoe UI", Helvetica, Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol"; */
        font-family: "IBM Plex Serif", Georgia, Cambria, "Times New Roman", serif;
        font-size: 1.05rem;
    }

    h1, h2, h3, h4, h5, h6 {
        font-family: "IBM Plex Serif", Georgia, Cambria, "Times New Roman", serif;
    }

    code, kbd, pre, textarea {
        font-family: "IBM Plex Mono", Menlo, Consolas, Monaco, "Liberation Mono", "Lucida Console", monospace;
        font-size: 1rem;
    }

    div.sphinxsidebar {
        font-size: 1rem;
    }

    nav#contents {
        margin-left: 1em;
        padding-bottom: 1em;
        padding-left: 1em;
        width: 90%;
    }

    p.caption, .caption-text {
        font-weight: bold;
    }

    .bolditalic {
        font-style: italic;
        font-weight: bold;
    }

    /* Hide "On GitHub" section from versions menu */
    div.injected > div.rst-versions > div.rst-other-versions > dl:nth-child(4) {
        display: none;
    }
}
